#!/usr/bin/env python3
"""
ARRAY_CONTAINS Executor
Company: eXonware.com
Author: eXonware Backend Team
Email: connect@exonware.com
Version: 0.9.0.6
"""

from typing import Any, Optional
from ..base import AUniversalOperationExecutor
from ....contracts import QueryAction, ExecutionContext, ExecutionResult
from ....defs import OperationType
from ..utils import extract_items, extract_field_value


class ArrayContainsExecutor(AUniversalOperationExecutor):
    """ARRAY_CONTAINS - Check whether an array field contains a value."""
    OPERATION_NAME = "ARRAY_CONTAINS"
    OPERATION_TYPE = OperationType.ARRAY
    SUPPORTED_NODE_TYPES = []

    def _do_execute(self, action: QueryAction, context: ExecutionContext) -> ExecutionResult:
        params = action.params
        result_data = self._execute_op(context.node, params, context)
        return ExecutionResult(
            success=True,
            data=result_data,
            action_type=self.OPERATION_NAME,
            metadata={"operation": self.OPERATION_NAME},
        )

    def _execute_op(self, node: Any, params: dict, context: ExecutionContext) -> dict:
        field: Optional[str] = params.get("field")
        value: Any = params.get("value")
        as_field: str = params.get("as_field", "_array_contains")
        items = extract_items(node)

        result_items = []
        for item in items:
            item_copy = item.copy() if isinstance(item, dict) else {"value": item}
            array_value = extract_field_value(item, field) if field else None
            item_copy[as_field] = value in array_value if isinstance(array_value, list) else False
            result_items.append(item_copy)

        return {
            "items": result_items,
            "count": len(result_items),
            "total_items": len(items),
            "field": field,
            "value": value,
            "as_field": as_field,
        }


__all__ = ["ArrayContainsExecutor"]
