#!/usr/bin/env python3
"""
Parser Contracts
Company: eXonware.com
Author: eXonware Backend Team
Email: connect@exonware.com
Version: 0.9.0.6
Generation Date: October 26, 2025
"""
# Import from root contracts.py

from ...contracts import IParamExtractor
# Re-export
__all__ = ['IParamExtractor']
